@@ -1,6 +1,9 @@
-__version__ = '2.2.1'
+__version__ = '2.2.2'
 
 VERSION = __version__  # synonym
 
 # Header encoding (see RFC5987)
 HTTP_HEADER_ENCODING = 'iso-8859-1'
+
+# Default datetime input and output formats
+ISO_8601 = 'iso-8601'
